﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;

namespace CustomSorter
{
    public class Sorter<T, TResult> where T : class
    {
        internal List<Expression<Func<T, TResult>>> _propertyList = new List<Expression<Func<T, TResult>>>();
        internal List<string> _originalOrderList = new List<string>();
        internal List<StringComparison> _comparisonList = new List<StringComparison>();

        private Dictionary<string, List<string>> _propertyOrderDict;
        private Dictionary<string, StringComparison> _propertyComparisonDict;

        public Sorter<T, TResult> Add(Expression<Func<T, TResult>> propertyList, string order)
        {
            return Add(propertyList, order, StringComparison.CurrentCulture);
        }

        public Sorter<T, TResult> Add(Expression<Func<T, TResult>> propertyList, string order, StringComparison comparison)
        {
            _propertyList.Add(propertyList);
            _originalOrderList.Add(order);
            _comparisonList.Add(comparison);
            return this;
        }

        internal void Sort(IList<T> source)
        {
            PopulatePropertyOrderDict(source);
            if (_propertyOrderDict.Count > 0) RecursiveSort(source, 0);
        }

        private int _recursiveCounter = 0;
        internal void RecursiveSort(IList<T> source, int nextLevel)
        {
            var propertyName = _propertyOrderDict.Keys.ToList()[_recursiveCounter + nextLevel];
            var filteredOrderList = _propertyOrderDict[propertyName];
            var comparison = _propertyComparisonDict[propertyName];

            var orderedObjectsList = new List<List<T>>();

            var index = 0;
            foreach (var filteredOrder in filteredOrderList)
            {
                var orderedObjects = new List<T>();

                while (source.FirstOrDefault(x => x != null && string.Compare(ReflectionHelper.GetValue(x, propertyName), filteredOrder, comparison) == 0) != null)
                {
                    var obj = source.First(x => x != null && string.Compare(ReflectionHelper.GetValue(x, propertyName), filteredOrder, comparison) == 0);
                    source.Remove(obj);
                    orderedObjects.Add(obj);
                }

                if (orderedObjects.Count > 0) orderedObjectsList.Add(orderedObjects);
                index++;
            }

            var needSubSort = false;
            foreach (var orderedObjects in orderedObjectsList)
            {
                //recursive call if there are more than 1 object with the same first order sort, and more than 1 sort is supplied
                if (orderedObjects.Count() > 1 && _propertyOrderDict.Keys.Count > _recursiveCounter + 1)
                {
                    RecursiveSort(orderedObjects, 1);
                    needSubSort = true;
                }
            }

            if (needSubSort) _recursiveCounter++;

            var i = 0;
            foreach (var orderedObjects in orderedObjectsList)
            {
                foreach (var obj in orderedObjects)
                {
                    source.Insert(i++, obj);
                }
            }
        }

        private void PopulatePropertyOrderDict(IList<T> source)
        {
            _propertyOrderDict = new Dictionary<string, List<string>>();
            _propertyComparisonDict = new Dictionary<string, StringComparison>();

            var index = 0;
            foreach (var property in _propertyList)
            {
                var propertyName = ReflectionHelper.GetMemberInfo(property);
                var comparison = _comparisonList[index];

                //get rid of empty entris and entries that do not exist in a property value list
                var refinedOrderList = FilterOrderList(source, _originalOrderList[index], propertyName, comparison);
                if (refinedOrderList.Count() > 0)
                {
                    _propertyOrderDict.Add(propertyName, refinedOrderList);
                    _propertyComparisonDict.Add(propertyName, comparison);
                }
                index++;
            }
        }

        private List<string> FilterOrderList(IList<T> source, string orderList, string propertyName, StringComparison comparison)
        {
            var refinedOrderList = new List<string>();
            foreach (var orderElement in orderList.Split(new[] { "," }, StringSplitOptions.RemoveEmptyEntries))
            {
                var val = orderElement;
                if (source.Any(x => x != null && string.Compare(val, ReflectionHelper.GetValue(x, propertyName), comparison) == 0))
                {
                    refinedOrderList.Add(orderElement);
                }
            }

            return refinedOrderList;
        }
    }
}
