﻿
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Web.UI;
using System.Web.UI.WebControls;
using DomainModel;
using Rhino.Mocks;
using Xunit.Extensions;

namespace GenericGridViewHelper.Specifications.DataSourceHelperSpec
{
    public class When_data_source_is_a_generic_list_and_no_sort_column : Specification
    {
        private int _totalRowCount;
        private IEnumerable _dataSource;
        private List<Product> _originalSource;

        public override void Observe()
        {
            var viewState = new StateBag();

            _originalSource = new List<Product> { 
                new Product { Name = "a", Price = 10 }, 
                new Product { Name = "b", Price = 20 } 
            };

            _dataSource = _originalSource;

            DataSourceHelper.PrepareDataSource(new GridView(), viewState, ref _dataSource, ref _totalRowCount, null);
        }

        [Observation]
        public void should_have_no_change_to_the_data_source_and_get_the_total_row_count_only()
        {
            _totalRowCount.ShouldEqual(_originalSource.Count);

            _dataSource.ShouldBeSameAs(_originalSource); //checking reference equal
        }
    }

    public class When_data_source_is_an_IEnumerable_and_no_sort_column : Specification
    {
        private int _totalRowCount;
        private IEnumerable _dataSource;
        private List<Product> _originalSource;

        public override void Observe()
        {
            var viewState = new StateBag();
            _originalSource = new List<Product> { 
                new Product { Name = "a", Price = 10 }, 
                new Product { Name = "b", Price = 20 } 
            };

            _dataSource = from x in _originalSource select x;

            DataSourceHelper.PrepareDataSource(new GridView(), viewState, ref _dataSource, ref _totalRowCount, null);
        }

        [Observation]
        public void should_convert_the_data_source_to_generic_list_and_get_the_total_row_count()
        {
            _totalRowCount.ShouldEqual(_originalSource.Count);

            _dataSource.GetType().Name.ShouldEqual(Const.ListTypeName);
        }
    }

    public class When_data_source_is_an_IEnumerable_and_sort_column_exists_in_view_state : Specification
    {
        private int _totalRowCount;
        private IEnumerable _dataSource;
        private List<Product> _originalSource;

        public override void Observe()
        {
            var viewState = new StateBag();
            viewState[Const.SortExpression] = "Name";

            _originalSource = new List<Product> { 
                new Product { Name = "b", Price = 20 }, 
                new Product { Name = "a", Price = 10 }
            };

            _dataSource = from x in _originalSource select x;

            DataSourceHelper.PrepareDataSource(new GridView(), viewState, ref _dataSource, ref _totalRowCount, null);
        }

        [Observation]
        public void should_convert_the_data_source_to_sorted_list_and_get_the_total_row_count()
        {
            _totalRowCount.ShouldEqual(_originalSource.Count);

            _dataSource.GetType().Name.ShouldEqual(Const.ListTypeName);

            _dataSource.Cast<Product>().ToList()[0].Name.ShouldEqual("a");
            _dataSource.Cast<Product>().ToList()[1].Name.ShouldEqual("b");
        }
    }

    public class When_sorted_data_source_is_null : Specification
    {
        private CompositeDataBoundControl _gridView;
        public override void Observe()
        {
            _gridView = MockRepository.GenerateStub<CompositeDataBoundControl>();
            var orginalDataSource = new List<Product> { new Product { Name = "b" }, new Product { Name = "a" } };
            IEnumerable sortedDataSource = null;
            var args = new GridViewSortEventArgs("Name", SortDirection.Ascending);

            DataSourceHelper.SelectDataSourceAndBind(_gridView, sortedDataSource, orginalDataSource, args);
        }

        [Observation]
        public void Should_sort_the_original_data_source_and_bind()
        {
            ((Product)((List<object>)_gridView.DataSource)[0]).Name.ShouldEqual("a");
            ((Product)((List<object>)_gridView.DataSource)[1]).Name.ShouldEqual("b");

            _gridView.AssertWasCalled(x => x.DataBind());
        }
    }

    public class When_sorted_data_source_is_not_null_and_is_not_a_list : Specification
    {
        private CompositeDataBoundControl _gridView;
        public override void Observe()
        {
            _gridView = MockRepository.GenerateStub<CompositeDataBoundControl>();
            IEnumerable orginalDataSource = null;
            IEnumerable sortedDataSource =
                from x in new List<Product> { new Product { Name = "b" }, new Product { Name = "a" } }
                select x;
            var args = new GridViewSortEventArgs("Name", SortDirection.Ascending);

            DataSourceHelper.SelectDataSourceAndBind(_gridView, sortedDataSource, orginalDataSource, args);
        }

        [Observation]
        public void Should_convert_sorted_data_source_to_list_and_bind()
        {
            _gridView.DataSource.GetType().Name.ShouldEqual(Const.ListTypeName);
            _gridView.AssertWasCalled(x => x.DataBind());
        }
    }
}
